package org.wso2.registry.app;


import org.wso2.registry.Comment;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.Tag;

import java.util.Properties;
import java.util.List;
import java.util.ArrayList;


public class ResourceHandling extends AbstractAPPTest {

    
    public void testResourceCopy() {

        try {

            String path="/f59/f2/r1";
            String commentPath = path + ";comments";
            String new_path="/f60/f2/r2";
            String commentPathNew = new_path + ";comments";
            Resource r1 = registry.newResource();
            r1.setDescription("This is a file to be renamed");
            byte[] r1content = "R2 content".getBytes();
            r1.setContent(r1content);
            r1.setMediaType("txt");

            Comment c1 = new Comment();
            c1.setResourcePath(path);
            c1.setText("This is a test comment1");

            Comment c2 = new Comment();
            c2.setResourcePath(path);
            c2.setText("This is a test comment2");

            Properties r1Pro = new Properties();
            r1Pro.setProperty("key1", "value1");
            r1Pro.setProperty("key2", "value2");
            r1.setProperties(r1Pro);

            registry.put(path,r1);
            registry.addComment(path,c1);
            registry.addComment(path,c2);
            registry.applyTag(path, "tag1");
            registry.applyTag(path, "tag2");
            registry.applyTag(path, "tag3");
            registry.rateResource(path, 4);

            Resource r2 = registry.get(path);

            assertEquals("Properties are not equal",r1.getProperty("key1"),
                         r2.getProperty("key1"));

            assertEquals("Properties are not equal",r1.getProperty("key2"),
                         r2.getProperty("key2"));

            assertEquals("File content is not matching", new String((byte[])r1.getContent()),
                     new String((byte[])r2.getContent()));

            assertTrue(c1.getText() + " is not associated for resource"+ path,
                        containsComment(commentPath,c1.getText()));

            assertTrue(c2.getText() + " is not associated for resource"+ path,
                        containsComment(commentPath,c2.getText()));

            assertTrue("Tag1 is not exist",containsTag(path,"tag1"));
            assertTrue("Tag2 is not exist",containsTag(path,"tag2"));
            assertTrue("Tag3 is not exist",containsTag(path,"tag3"));

            float rating = registry.getAverageRating(path);
            assertEquals("Rating is not mathching", rating, (float)4.0,
                        (float)0.01);

            assertEquals("Media type not exist",r1.getMediaType(),
                        r2.getMediaType());

            assertEquals("Authour name is not exist",r1.getAuthorUserName(),
                        r2.getAuthorUserName());

            assertEquals("Description is not exist",r1.getDescription(),
                        r2.getDescription());

            registry.rename(path,new_path);
            
            /*get renamed resource details*/
            Resource r1Renamed = registry.get(new_path);

            assertEquals("File content is not matching", new String((byte[])r2.getContent()),
                     new String((byte[])r1Renamed.getContent()));

            assertEquals("Properties are not equal",r2.getProperty("key1"),
                        r1Renamed.getProperty("key1"));

            assertEquals("Properties are not equal",r2.getProperty("key2"),
                        r1Renamed.getProperty("key2"));

            assertTrue(c1.getText() + " is not associated for resource"+ new_path,
                        containsComment(commentPathNew,c1.getText()));

            assertTrue(c2.getText() + " is not associated for resource"+ new_path,
                        containsComment(commentPathNew,c2.getText()));

            assertTrue("Tag1 is not copied",containsTag(new_path,"tag1"));
            assertTrue("Tag2 is not copied",containsTag(new_path,"tag2"));
            assertTrue("Tag3 is not copied",containsTag(new_path,"tag3"));

            float rating1 = registry.getAverageRating(new_path);
            assertEquals("Rating is not copied", rating1, (float)4.0,
                        (float)0.01);

            assertEquals("Media type not copied",r2.getMediaType(),
                        r1Renamed.getMediaType());

            assertEquals("Authour Name is not copied",r2.getAuthorUserName(),
                        r1Renamed.getAuthorUserName());

            assertEquals("Description is not exist",r2.getDescription(),r1Renamed.getDescription());

            }catch (RegistryException e) {
                       e.printStackTrace();
            }
    }

    public void testCollectionCopy() {

        try {

            String path="/c901/c1/c2";
            String commentPath = path + ";comments";
            String new_path="/c911/c1/c3";
            String commentPathNew = new_path + ";comments";
            Resource r1 = registry.newResource();
            r1.setDescription("This is a file to be renamed");

            Comment c1 = new Comment();
            c1.setResourcePath(path);
            c1.setText("This is first test comment");

            Comment c2 = new Comment();
            c2.setResourcePath(path);
            c2.setText("This is secound test comment");

            Properties r1Pro = new Properties();
            r1Pro.setProperty("key1", "value1");
            r1Pro.setProperty("key2", "value2");
            r1.setProperties(r1Pro);

            registry.put(path,r1);
            registry.addComment(path,c1);
            registry.addComment(path,c2);
            registry.applyTag(path, "tag1");
            registry.applyTag(path, "tag2");
            registry.applyTag(path, "tag3");
            registry.rateResource(path, 4);

            Resource r2 = registry.get(path);

            assertEquals("Properties are not equal",r1.getProperty("key1"),
                         r2.getProperty("key1"));

            assertEquals("Properties are not equal",r1.getProperty("key2"),
                         r2.getProperty("key2"));

            assertTrue(c1.getText() + " is not associated for resource"+ path,
                        containsComment(commentPath,c1.getText()));

            assertTrue(c2.getText() + " is not associated for resource"+ path,
                        containsComment(commentPath,c2.getText()));

            assertTrue("Tag1 is not copied",containsTag(path,"tag1"));
            assertTrue("Tag2 is not copied",containsTag(path,"tag2"));
            assertTrue("Tag3 is not copied",containsTag(path,"tag3"));

            float rating = registry.getAverageRating(path);
            assertEquals("Rating is not mathching", rating, (float)4.0,
                        (float)0.01);
            assertEquals("Authour name is not exist",r1.getAuthorUserName(),r2.getAuthorUserName());

            registry.rename(path,new_path);

            /*get renamed resource details*/

            Resource r1Renamed = registry.get(new_path);

            assertEquals("Properties are not equal",r2.getProperty("key1"),
                        r1Renamed.getProperty("key1"));

            assertEquals("Properties are not equal",r2.getProperty("key2"),
                        r1Renamed.getProperty("key2"));

            assertTrue(c1.getText() + " is not associated for resource"+ new_path,
                        containsComment(commentPathNew,c1.getText()));

            assertTrue(c2.getText() + " is not associated for resource"+ new_path,
                        containsComment(commentPathNew,c2.getText()));

            assertTrue("Tag1 is not copied",containsTag(new_path,"tag1"));
            //assertTrue("Tag2 is not copied",containsTag(new_path,"tag2"));
            //assertTrue("Tag3 is not copied",containsTag(new_path,"tag3"));

            float rating1 = registry.getAverageRating(new_path);
            assertEquals("Rating is not copied", rating1, (float)4.0,
                        (float)0.01);

            assertEquals("Author Name is not copied",r1.getAuthorUserName(),
                        r2.getAuthorUserName());

            }catch (RegistryException e) {
                       e.printStackTrace();
            }
    }

    private boolean containsComment(String pathValue, String commentText) {

        Comment[] commentsArray = null;
        List commentTexts = new ArrayList();

        try{
            Resource commentsResource = registry.get(pathValue);
            commentsArray = (Comment[])commentsResource.getContent();
            for (Comment comment : commentsArray) {
                Resource commentResource = registry.get(comment.getCommentPath());
                commentTexts.add(commentResource.getContent());
            }
        }catch (RegistryException e) {
            e.printStackTrace();
        }

        boolean found = false;
        if(commentTexts.contains(commentText)){
            found=true;
        }

        return found;
    }

    private boolean containsTag(String tagPath,String tagText){

        Tag[] tags = null;

        try {
            tags = registry.getTags(tagPath);
        } catch (RegistryException e) {
            e.printStackTrace();
        }

        boolean tagFound = false;
        for (int i = 0; i < tags.length; i++) {
            if (tags[i].getTagName().equals(tagText)) {
                tagFound = true;
                break;
            }
        }

        return tagFound;
    }

    private boolean containsString(String[] array, String value) {

        boolean found = false;
        for (String anArray : array) {
            if (anArray.startsWith(value)) {
                found = true;
                break;
            }
        }

        return found;
    }

}

